# Movable Type plugin for comment order number.
#
# Release 1.00 (Oct 15, 2006) initial version.
#         1.01 (Oct 17, 2006) fix bug.
#         1.02 (Oct 21, 2006) fix bug.
#         1.03 (Oct 25, 2006) add lastn attribute.
#         1.04 (Oct 27, 2006) add start_max attribute.
#
# This software is provided as-is. You may use it for commercial or
# personal use. If you distribute it, please keep this notice intact.
#
# Copyright (c) 2004-2006 Yujiro Araki

package MT::Plugin::CommentOrderNumberByDescend;
use strict;
use MT;
use MT::Template::Context;

use base qw(MT::Plugin);

our $VERSION = '1.04';

my $plugin = MT::Plugin::CommentOrderNumberByDescend->new({
    name => 'CommentOrderNumberByDescend',
    description => 'The placement of the comment in numerical order beginning at last.',
    doc_link => 'http://www.koikikukan.com/archives/2006/10/16-005050.php',
    author_name => 'Yujiro Araki',
    author_link => 'http://www.koikikukan.com/',
    version => $VERSION
});
MT->add_plugin($plugin);

MT::Template::Context->add_tag(CommentOrderNumberByDescend => \&_hdlr_comment_order_num_by_descend);

sub _hdlr_comment_order_num_by_descend {
    my $ctx = shift;
    my $args = shift;

    $ctx->stash('comment') or return $ctx->_no_comment_error('MTCommentDescendOrderNumber');

    # \Rg擾(lastnD)
    my $count = $ctx->stash('entry')->comment_count;

    # lastn + start_max̏ꍇ, lRg($count+$gap)ɂ.
    my $gap = 0;
    if ($args->{lastn}) {
        if ($args->{start_max}) {
            $gap = ($count - $args->{lastn} > 0) ? $count - $args->{lastn} : 0;
        }
        $count = $args->{lastn};
    }

    # Rgpf[^擾
    # Õev[glastn=1̏ꍇCommentIDNG
    my $id = $ctx->stash('comment')->id;
    my $old_id = $ctx->stash('comment_descend_order_num_id');

    # Rg̃^Odݒɂ鋫ElC
    my $pad = $old_id eq $id ? 0 : 1;

    # JEgZ
    if ($ctx->stash('comment_descend_order_num') > $pad) {
        $count = $ctx->stash('comment_descend_order_num') - 1;
    }

    # Rg̏ꍇ
    return ($count + 1 + $gap) if $old_id eq $id;

    # كRg̏ꍇ
    $ctx->stash('comment_descend_order_num_id', $id);
    $ctx->stash('comment_descend_order_num', $count);
    if ($args->{lastn} eq '1') {
        $ctx->stash('comment_descend_order_num', undef);
    }
    return ($count + $gap);
}

1;
